package dao

import models.TestCase

import scala.collection.mutable.ListBuffer

class TestCaseDAO {

  private var allTests : ListBuffer[TestCase] = loadTests()

  private def loadTests() = {
    val tests = scala.xml.XML.loadFile("data/test/tests.xml")
    (tests \ "test").map(test => {
      TestCase((test \ "@id").text,
        (test \ "name").text,
        (test \ "description").text,
        (test \ "request").text,
        (test \ "parameters").text.split(";").toSeq,
        (test \ "urn").text,
        scala.xml.XML.loadFile("data/test/" + (test \ "expected").text))
    }).to[ListBuffer]
  }

  def reload() = {
    allTests = loadTests()
  }

  def saveAllTests() = {

    allTests.foreach(test => {
      scala.xml.XML.save("data/test/test-" + test.id + ".xml", test.expectedResponse, "UTF-8", xmlDecl = true)
    })

    val allXmlTest = allTests.map(test => {
      <test id={test.id}>
        <name>{test.name}</name>
        <description>{test.description}</description>
        <expected>test-{test.id}.xml</expected>
        <request>{test.requestType}</request>
        <urn>{test.urn}</urn>
        <parameters>{test.requestParameters.mkString(";")}</parameters>
      </test>
    })
    val testSummaryDoc = <tests>{allXmlTest}</tests>
    scala.xml.XML.save("data/test/tests.xml", testSummaryDoc, "UTF-8", xmlDecl = true)
  }

  def all() : Seq[TestCase] = allTests

  def findById(id:String) : Option[TestCase] = {
    all().find(_.id == id)
  }

  def update(testCase: TestCase) = {
    allTests = allTests.filterNot(_.id == testCase.id) += testCase
    allTests = allTests.sortBy(_.id)

    saveAllTests()
  }

  def add(testCase: TestCase) = {
    allTests += testCase
    allTests = allTests.sortBy(_.id)

    saveAllTests()
  }
}
