package services

import java.util.Date

import models.{TestCase, TestResult, TestRun}
import play.api.Logger
import xmldiff.{Comparison, NoDiff}

class TestRunner {

  def run(tests:Seq[TestCase], baseUrl:String) : TestRun = {
    val startDate = new Date()

    val testResults = tests.zipWithIndex.map{ case (testCase, index) =>
      val url = buildUrl(testCase, baseUrl)

      try {
        val xml = scala.xml.XML.load(url)

        val comparison = new Comparison
        comparison.compare(testCase.expectedResponse, xml) match {
          case NoDiff => TestResult(index, testCase.name, testCase.description, new Date(), "passed", url, Some(xml.toString()), testCase.id)

          case diff => TestResult(index, testCase.name, testCase.description, new Date(), "failed", url, Some(diff.toString), testCase.id)
        }
      } catch {
        case _: Throwable =>
	        val response = "invalid response from server:"
          TestResult(index, testCase.name, testCase.description, new Date(), "failed", url, Some(response), testCase.id)
      }
    }
    TestRun(None, startDate, testResults)
  }

  def buildUrl(testCase: TestCase, baseUrl:String):String = {
    val url = s"$baseUrl?request=${testCase.requestType}&urn=${testCase.urn}"
    if (testCase.requestParameters.nonEmpty) {
      url + testCase.requestParameters.mkString("&", "&", "")
    } else {
      url
    }
  }
}
